open Common

type t = Leq of term * term
       | Eq of term * term
       | Not of t
       | Or of t * t
       | And of t * t
       | Forall of (var -> t)
       | Box of hp * t
and hp = Test of t
       | Assign of var * term
       | Evolve of Poly.Solve.t * t
       | Seq of hp * hp
       | Choice of hp * hp

module Syntax = struct
  let (??) t = Test t

  let (:=) x e = Assign(x, e)

  let (^=) x e = (x, e)

  let (&) e d = Evolve(e, d)

  let (^) a b = Seq(a, b)

  let (++) a b = Choice(a, b)

  let (<=) x y = Leq(x, y)

  let (=) x y = Eq(x, y)

  let tt = let open Poly.Multi.Syntax in !0. = !0.

  let (!=) x y = Not(Eq(x, y))

  let not t = Not t

  let ff = not tt

  let (>) x y = not (x <= y)

  let (||) x y = Or(x, y)

  let (>=) x y = x > y || x = y

  let (<) x y = not (x >= y)

  let (&&) x y = And(x, y)

  let (=>) a b = Not a || b

  let (<=>) a b = a => b && b => a

  let forall t = Forall(fun x -> t (Poly.Multi.Var x))

  let exists t = Not(forall (fun x -> Not (t x)))

  let (!|) a t = Box(a, t)

  let (|!) f x = f x

  let (!<) a t = Not(Box(a, Not t))

  let (>!) f x = f x
end
