(* Embedding of dL. See README for explanation. *)
open Common

type t = Leq of term * term
       | Eq of term * term
       | Not of t
       | Or of t * t
       | And of t * t
       | Forall of (var -> t) (* Weak HOAS *)
       | Box of hp * t
and hp = Test of t
       | Assign of var * term
       | Evolve of Poly.Solve.t * t
       | Seq of hp * hp
       | Choice of hp * hp

module Syntax : sig
  val (??) : t -> hp

  val (:=) : var -> term -> hp

  val (^=) : var -> term -> var * term

  val (&) : Poly.Solve.t -> t -> hp

  val (^) : hp -> hp -> hp

  val (++) : hp -> hp -> hp

  val tt : t

  val ff : t

  val (>) : term -> term -> t

  val (<) : term -> term -> t

  val (<=) : term -> term -> t

  val (>=) : term -> term -> t

  val (=) : term -> term -> t

  val (!=) : term -> term -> t

  val not : t -> t

  val (||) : t -> t -> t

  val (&&) : t -> t -> t

  val (=>) : t -> t -> t

  val (<=>) : t -> t -> t

  val forall : (term -> t) -> t

  val exists : (term -> t) -> t

  val (!|) : hp -> t -> t 

  val (|!) : ('a -> 'b) -> 'a -> 'b

  val (!<) : hp -> t -> t

  val (>!) : ('a -> 'b) -> 'a -> 'b
end
