open Core
open Poly__Multi
open AST
open AST.Syntax

(* Transforms {x'=f(x)} to x := x(t) where x'(t) = f(x(t)) *)
let assign_ode t es p = List.fold_left (Poly__Solve.solve t es)
  ~init:p ~f:(fun p (x, e) -> !| (x := e) |! p)

(* See term paper for details on the translation into FOL *)
let rec to_FOL = function
| Leq(l, r) -> FOL.Leq(l, r)
| Eq(l, r)  -> FOL.Eq(l, r)
| Not t     -> FOL.Not(to_FOL t)
| Or(l, r)  -> FOL.Or(to_FOL l, to_FOL r)
| And(l, r) -> FOL.And(to_FOL l, to_FOL r)
| Forall t  -> FOL.Forall(fun x -> to_FOL (t x))
| Box(a, p) -> begin match a with
  | Test q        -> FOL.implies (to_FOL q) (to_FOL p)
  | Assign(x, e)  -> FOL.subst x e (to_FOL p)
  | Seq(a, b)     -> to_FOL (!| a |! (!| b |! p))
  | Choice(a, b)  -> to_FOL ((!| a |! p) && (!| b |! p))
  | Evolve(es, q) -> to_FOL @@
      Forall (fun t -> Var t >= zero =>
        (Forall (fun s ->
          (zero <= Var s && Var s <= Var t) => assign_ode s es q)
        => assign_ode t es p))
  end

let check t = FOL.check (to_FOL t)
