open Core

type t = (Multi.var, Multi.t) List.Assoc.t

module G = Graph.Persistent.Digraph.ConcreteBidirectional(struct
  type t                    = Multi.var * Multi.t
  let compare (x, _) (y, _) = compare x y
  let equal   (x, _) (y, _) = x = y
  let hash                  = Hashtbl.hash
end)

module T = Graph.Traverse.Dfs(G)

module P = Graph.Path.Check(G)

open Multi
open Uni

let solve t es =
  (* x' = f -> y' = g iff x occurs in g. See paper for DAG syntactic check. *)
  let g = List.fold_left (List.cartesian_product es es) ~init:G.empty
    ~f:(fun g (((x, _) as s), ((_, f) as t)) ->
          if occurs_in x f then G.add_edge g s t else g) in
  (* Solve by back-sub if solutions are polynomial *)
  if T.has_cycle g then
    failwith "ODE with non-polynomial solution found."
  else let pc = P.create g in
    List.map ~f:(fun (x, f) -> x, to_term t f)
      (List.fold_left ~init:[] ~f:(fun s (x, f) ->
        (x, integrate (Var x) (of_term s f)) :: s)
        (List.sort es ~compare:(fun a b ->
           try  if P.check_path pc a b then -1
           else if P.check_path pc b a then  1
           else 0 with Invalid_argument _ -> 0)))
